/*
 * Copyright (C) 2022  Aravinth Manivannan <realaravinth@batsense.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
use serde::{Deserialize, Serialize};
use url::Url;

use crate::ctx::Ctx;
use crate::db;
use crate::db::Site;
use crate::errors::*;
use crate::page::Page;
use crate::settings::Settings;
use crate::utils::get_random;

#[derive(Clone, Debug, Deserialize, Serialize, Eq, PartialEq)]
/// Data required to add site
pub struct AddSite {
    pub url: Url,
    pub owner: String,
}

impl AddSite {
    fn to_site(self, s: &Settings) -> Site {
        let site_secret = get_random(32);
        Site {
            url: self.url,
            owner: self.owner,
        }
    }
}

impl Ctx {
    pub async fn add_site(&self, site: AddSite) -> ServiceResult<Page> {
        let db_site = site.to_site(&self.settings);
        self.db.add_site(&db_site).await.unwrap();
        let page = Page::from_site(&self.settings, db_site);
        page.archive(&self).await.unwrap();
        Ok(page)
    }
}
